--{{{ Libraries

local RL = AceLibrary("Roster-2.1")
local Aura = AceLibrary("SpecialEvents-Aura-2.0")
local L = AceLibrary("AceLocale-2.2"):new("GridStatusHots")

--}}}
--

local hottimer = { }
local playerClass, englishClass = UnitClass("player");
local hotspell=nil
local hotspellduration=nil
local gsh_target=nil
local dbsettings=nil

--{{{ Localization

L:RegisterTranslations("enUS", function()
	return {
		["My HoTs"] = true,
		["Renew"] = true,
		["Rejuvenation"] = true,
		["Regrowth"] = true,
		["Lifebloom"] = true,
		["Buff: Hot Count"] = true,
		["Buff: My Renew"] = true,
		["Buff: My Rejuvenation"] = true,
		["Buff: My Regrowth"] = true,
		["Buff: My Lifebloom"] = true,
		["Set duration"] = true,
		["Set duration of your HoT"] = true,
	}
end)

L:RegisterTranslations("deDE", function()
	return {
		["My HoTs"] = "Meine HoTs",
		["Renew"] = "Erneuerung",
		["Rejuvenation"] = "Verj\195\188ngung",
		["Regrowth"] = "Nachwachsen",
		["Lifebloom"] = "Bl\195\188hendes Leben",
		["Buff: Hot Count"] = "Anzahl Hots",
		["Buff: My Renew"] = "Meine Erneuerung",
		["Buff: My Rejuvenation"] = "Meine Verj\195\188ngung",
		["Buff: My Regrowth"] = "Mein Nachwachsen",
		["Buff: My Lifebloom"] = "Mein Bl\195\188hendes Leben",
		["Set duration"] = "Set Duration",
		["Set duration of your HoT"] = "Set duration of your HoT",
}
end)

L:RegisterTranslations("koKR", function() 
	return {
		["My HoTs"] = "나의 지속 치유",
		["Renew"] = "소생",
		["Rejuvenation"] = "회복",
		["Regrowth"] = "재생",
		["Lifebloom"] = "피어나는 생명",
		["Buff: Hot Count"] = "버프: 지속 갯수",
		["Buff: My Renew"] = "버프 : 나의 소생",
		["Buff: My Rejuvenation"] = "버프 : 나의 회복",
		["Buff: My Regrowth"] = "버프 : 나의 재생",
		["Buff: My Lifebloom"] = "버프: 나의 피어나는 생명",
		["Set duration"] = "Set Duration",
		["Set duration of your HoT"] = "Set duration of your HoT",
	}
end)

L:RegisterTranslations("frFR", function() 
	return {
		["My HoTs"] = "My HoTs", --need translation
		["Renew"] = "R\195\169novation",
		["Rejuvenation"] = "R\195\169cup\195\169ration",
		["Regrowth"] = "R\195\169tablissement",
		["Lifebloom"] = "Lifebloom", -- need translation
		["Buff: Hot Count"] = "Buff: Hot Count", --need translation
		["Buff: My Renew"] = "Buff: My Renew", --need translation
		["Buff: My Rejuvenation"] = "Buff: My Rejuvenation", --need translation
		["Buff: My Regrowth"] = "Buff: My Regrowth", --need translation
		["Buff: My Lifebloom"] = "Buff: My Lifebloom", --need translation
		["Set duration"] = "Set Duration",
		["Set duration of your HoT"] = "Set duration of your HoT",
	}
end)

L:RegisterTranslations("zhCN", function() 
	return {
		["My HoTs"] = "My HoTs", --need translation
		["Renew"] = "\230\191\162\229\164\191",
		["Rejuvenation"] = "\229\155\158\230\152\165\230\156\175",
		["Regrowth"] = "\230\132\136\229\191\136",
		["Lifebloom"] = "Lifebloom", -- need translation
		["Buff: Hot Count"] = "Buff: Hot Count", --need translation
		["Buff: My Renew"] = "Buff: My Renew", --need translation
		["Buff: My Rejuvenation"] = "Buff: My Rejuvenation", --need translation
		["Buff: My Regrowth"] = "Buff: My Regrowth", --need translation
		["Buff: My Lifebloom"] = "Buff: My Lifebloom", --need translation
		["Set duration"] = "Set Duration",
		["Set duration of your HoT"] = "Set duration of your HoT",
	}
end)

--}}}

GridStatusHots = GridStatus:NewModule("GridStatusHots")
GridStatusHots.menuName = L["My HoTs"]

--{{{ AceDB defaults
--
GridStatusHots.defaultDB = {
	alert_renew = {
		text = L["Buff: My Renew"],
		desc = L["Buff: My Renew"],
		enable = true,
		priority = 99,
		range = false,
		threshold = 15,
		color = { r = 0, g = 1, b = 0, a = 1 },
	},
	alert_rejuv = {
		text = L["Buff: My Rejuvenation"],
		desc = L["Buff: My Rejuvenation"],
		enable = true,
		priority = 99,
		range = false,
		threshold = 12,
		color = { r = 0, g = 0, b = 1, a = 1 },
	},
	alert_regrow = {
		text = L["Buff: My Regrowth"],
		desc = L["Buff: My Regrowth"],
		enable = true,
		priority = 98,
		range = false,
		threshold = 21,
		color = { r = 1, g = 1, b = 1, a = 1 },
	},
	alert_lifebl = {
		text = L["Buff: My Lifebloom"],
		desc = L["Buff: My Lifebloom"],
		enable = true,
		priority = 99,
		range = false,
		threshold = 7,
		color = { r = 1, g = 0, b = 1, a = 1 },
	},
	alert_tothots = {
		text = L["Buff: Hot Count"],
		desc = L["Buff: Hot Count"],
		enable = true,
		priority = 97,
		range = false,
		color = { r = 0, g = 1, b = 0, a = 1 },
	},
}



--}}}

local rejuv_hot_duration = {
	["threshold"] = {
		type = "range",
		name = L["Set duration"],
		desc = L["Set duration of your HoT"],
		max = 20,
		min = 12,
		step = 1,
		get = function ()
			      return GridStatusHots.db.profile.alert_rejuv.threshold
		      end,
		set = function (v)
			       GridStatusHots.db.profile.alert_rejuv.threshold = v
		      end,
	},
}
local regrow_hot_duration = {
	["threshold"] = {
		type = "range",
		name = L["Set duration"],
		desc = L["Set duration of your HoT"],
		max = 29,
		min = 21,
		step = 1,
		get = function ()
			      return GridStatusHots.db.profile.alert_regrow.threshold
		      end,
		set = function (v)
			       GridStatusHots.db.profile.alert_regrow.threshold = v
		      end,
	},
}
local lifebl_hot_duration = {
	["threshold"] = {
		type = "range",
		name = L["Set duration"],
		desc = L["Set duration of your HoT"],
		max = 9,
		min = 5,
		step = 1,
		get = function ()
			      return GridStatusHots.db.profile.alert_lifebl.threshold
		      end,
		set = function (v)
			       GridStatusHots.db.profile.alert_lifebl.threshold = v
		      end,
	},
}
local renew_hot_duration = {
	["threshold"] = {
		type = "range",
		name = L["Set duration"],
		desc = L["Set duration of your HoT"],
		max = 22,
		min = 15,
		step = 1,
		get = function ()
			      return GridStatusHots.db.profile.alert_renew.threshold
		      end,
		set = function (v)
			       GridStatusHots.db.profile.alert_renew.threshold = v
		      end,
	},
}

function GridStatusHots:OnInitialize()
	self.super.OnInitialize(self)
	
	self:RegisterStatus("alert_tothots", L["Buff: Hot Count"], alert_tothots)
	if englishClass == "PRIEST" then
		self:RegisterStatus("alert_renew", L["Buff: My Renew"], renew_hot_duration)
	end
	if englishClass == "DRUID" then
		self:RegisterStatus("alert_rejuv", L["Buff: My Rejuvenation"], rejuv_hot_duration)
		self:RegisterStatus("alert_regrow", L["Buff: My Regrowth"], regrow_hot_duration)
		self:RegisterStatus("alert_lifebl", L["Buff: My Lifebloom"], lifebl_hot_duration)
	end
end

function GridStatusHots:OnEnable()
	local f = function()
		GridStatusHots:UpdateAll()
	end
	self:RegisterEvent("UNIT_SPELLCAST_SENT")
	self:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED", "UpdateHots")
	self:ScheduleRepeatingEvent("GHS_Refresh", f, 1) 
end

function GridStatusHots:Reset()
	self.super.Reset(self)
end

function GridStatusHots:UNIT_SPELLCAST_SENT(unit, spell, rank, target)
	-- Did you cast a HoT ?
	if string.find(target,"-") then
		local intEnd = string.find(target, "-");
		target = string.sub(target,1,intEnd - 1);
	end
	if spell == L["Renew"] then
		gsh_target = target
		hotspell = L["Renew"]
		hotspellduration = self.db.profile.alert_renew.threshold
		cstatus = "alert_renew"
		dbsettings=self.db.profile.alert_renew
	elseif spell == L["Rejuvenation"] then
		gsh_target = target
		hotspell = L["Rejuvenation"]
		hotspellduration = self.db.profile.alert_rejuv.threshold
		cstatus = "alert_rejuv"
		dbsettings=self.db.profile.alert_rejuv
	elseif spell == L["Regrowth"] then
		gsh_target = target
		hotspell = L["Regrowth"]
		hotspellduration = self.db.profile.alert_regrow.threshold
		cstatus = "alert_regrow"
		dbsettings=self.db.profile.alert_regrow
	elseif spell == L["Lifebloom"] then
		gsh_target = target
		hotspell = L["Lifebloom"]
		hotspellduration = self.db.profile.alert_lifebl.threshold
		cstatus = "alert_lifebl"
		dbsettings=self.db.profile.alert_lifebl
	else
		gsh_target = nil
	end
end

function GridStatusHots:UpdateHots(target)
	-- Time your HoT if spellcast was successful.
	if gsh_target and hotspell == L["Regrowth"] then
	        hottimer[gsh_target.."3"] = { GetTime(), hotspellduration, cstatus, hotspell, dbsettings  }    
	elseif gsh_target and hotspell == L["Lifebloom"] then
		hottimer[gsh_target.."2"] = { GetTime(), hotspellduration, cstatus, hotspell, dbsettings  }
	elseif gsh_target then -- Renew and Rejuv use same timer, since they cant be cast both by you.
		hottimer[gsh_target.."1"] = { GetTime(), hotspellduration, cstatus, hotspell, dbsettings  }
	end
end

function GridStatusHots:UpdateAll()
	for u in RL:IterateRoster() do
		local name = UnitName(u.unitid) or ""
		local recnt = 0
		local rgcnt = 0
		local rjcnt = 0
		local lbcnt = 0
		local hcnttxt = nil
		-- Find number of hots pr unit
		for i=1,40 do -- this is not raid numbers, its possible buffs/debuffs to scan per member,
			local bname,brank,btexture,bcount = UnitBuff(u.unitid, i)
			if not bname then break end
			if bname and bname == L["Renew"] then recnt = recnt + 1 end
			if bname and bname == L["Regrowth"] then rgcnt = rgcnt + 1 end
			if bname and bname == L["Rejuvenation"] then rjcnt = rjcnt + 1 end
			if bname and bname == L["Lifebloom"] then lbcnt = lbcnt + 1 end
		end
		-- Set number of hots currently running on all units
		if recnt > 0 or rjcnt > 0 or rgcnt > 0 or lbcnt > 0 then
			hcnttxt = tostring(recnt+rjcnt+rgcnt+lbcnt)
			local settings = self.db.profile.alert_tothots
			self.core:SendStatusGained(name, "alert_tothots",
				settings.priority,
				(settings.range and 40),
				settings.color,
				tostring(hcnttxt)
			)
		else
			self.core:SendStatusLost(name, "alert_tothots")
		end
		
		for z=1,3 do
			-- Add status and countdown for self thrown HoTs
			if hottimer[name..z] then
				local settings = hottimer[name..z][5]
				local text = hottimer[name..z][2]-(GetTime()-hottimer[name..z][1])
				self.core:SendStatusGained(name, hottimer[name..z][3],
					settings.priority,
					(settings.range and 40),
					settings.color,
					tostring(math.floor(text).."-"..tostring(recnt+rjcnt+rgcnt+lbcnt))
				)
			end
			-- Remove status and countdown on self thrown Hots when expired
			if hottimer[name..z] and (GetTime()-hottimer[name..z][1]) > 1 then
				if (GetTime()-hottimer[name..z][1]) > hottimer[name..z][2] or not (Aura:UnitHasBuff(u.unitid, hottimer[name..z][4])) then
					self.core:SendStatusLost(name, hottimer[name..z][3])
				        hottimer[name..z] = nil
				end
			end
		end
	end
end
